# coding: utf-8

# 
# Dependences
# 
import wmi, socket, sys

# 
# Manipulador para obter nome da máquina e usuário logado
# 
class HUNames():

    def __init__ (
        self
    ):
        self.set_hostname()
        self.set_username()
        self.set_app()
    
    # 
    # obter nome da máquina
    # 
    def set_hostname (
        self
    ):
        try:
            
            try:
                self.hostname = socket.gethostname()
            
            except:
                self.hostname = socket.gethostbyname (
                    socket.gethostname()
                )
                pass
        
        except:
            self.hostname = 'N/D'
            pass

    def get_hostname (
        self
    ):
        return self.hostname

    # 
    # obter nome do usuário windows atualmente conectado
    # 
    def set_username (
        self
    ):
        try:
            user = list()
            wmi_object = wmi.WMI (
                self.hostname
            )
            user = wmi_object.Win32_LogonSession()[0].references (
                'Win32_LoggedOnUser'
            )
            if str (
                user[0].Antecedent.Name
            ) == 'SISTEMA':
                self.username = 'Administrador'
            else:
                self.username = str (
                    user[0].Antecedent.Name
                )
        
        except:
            self.username = 'N/D'
            pass

    def get_username (
        self
    ):
        return self.username

    # 
    # Obter a versão do Python instalado no sistema
    # 
    def set_app (
        self
    ):
        try:
            self.app_version = \
            'Python {}.{}'.format (
                sys.version_info.major
            ,   sys.version_info.minor
            )

        except:
            self.app_version = 'N/D'

    def get_app (
        self
    ):
        return self.app_version


# coding: utf-8

#
# Dependences
# 
import os, os.path, tempfile, shutil
import fnmatch      # fassad, 2020-06-30: adicionado biblioteca para busca de arquivos e diretorios
import string, random   # fassad, 2020-06-30: adicionado biblioteca para criação de diretórios com nomes randômicos

# 
# Obter caminho dos arquivos temporários
# 
class Files():
    
    def __init__ (
        self
    ,   folder_path=None
    ):
        self.set_folder_path (
            folder_path
        )
    
    # 
    # Procedimento para verificar a existência da pasta e set_up
    # 
    def set_folder_path (
        self
    ,   folder_path=None
    ):

        if folder_path:
            folder = folder_path
        else:
            # fassad, 2020-06-30: adicionado _random_path_name para criação de diretórios com nomes randômicos
            _random_path_name = ''.join(
                random.choice (
                    string.ascii_lowercase + string.digits
                ) for _ in range(6)
            )
            folder = str (
                tempfile.gettempdir()
            ) + '\\' + _random_path_name + '\\'

        if not os.path.isdir (
            folder
        ):                  # Return True if path is an existing directory
            os.mkdir(
                folder
            )               # Make diretory if not existing

        # Return the canonical path of the specified filename, eliminating any symbolic links encountered in the path
        self.folder_path = os.path.realpath (
            folder
        )
    
    # 
    # Retorna a pasta verificada
    # 
    def get_folder_path (
        self
    ):
        return self.folder_path
    
    # 
    # Excluir todos os arquivos da pasta
    # 
    def delete_all_files (
        self
    ):
        for file in os.listdir (
            self.folder_path
        ):
            os.remove (
                str (
                    self.folder_path
                ) + '\\' + str (
                    file
                )
            )

    # 
    # Excluir um arquivo temporário
    # 
    def delete_file (
        self
    ,   file_name
    ):
        os.remove (
            str (
                self.folder_path
            ) + '\\' + str (
                file_name
            )
        )

    # 
    # Listar arquivos do diretório
    # 
    def list_files (
        self
    ):
        return os.listdir (
            self.folder_path
        )

    # 
    # Mover arquivos de um diretório para outro
    # 
    def move_all_files (
        self
    ,   to_folder
    ):
        
        to_folder_valid = os.path.realpath (
            to_folder
        )

        for file in os.listdir (
            self.folder_path
        ):
            _old = self.folder_path + '\\' + str (file)
            _new = str (to_folder_valid) + '\\' + str (file)
            shutil.move (_old, _new)

    # 
    # Mover um arquivo específico de um diretório para outro
    # 
    def move_file (
        self
    ,   file_name
    ,   to_folder
    ):
        
        to_folder_valid = os.path.realpath (
            to_folder
        )

        _old = self.folder_path + '\\' + str (file_name)
        _new = str(to_folder_valid) + '\\' + str (file_name)
        shutil.move(_old, _new)

    # 
    # Buscar arquivos neste diretório
    # fassad, 2020-06-30: adicionado biblioteca para busca de arquivos e diretorios
    # 
    def search_path_or_file (
        self
    ,   name_or_pattern_to_search='*'
    ):
        
        result = []
        
        for root, dirs, files in os.walk (
            self.folder_path
        ):
            for file in files:
                if fnmatch.fnmatch (
                    file
                ,   name_or_pattern_to_search
                ):
                    result.append (
                        os.path.join (
                            root
                        ,   file
                        )
                    )
        
        return result


# coding: utf-8

#
# Dependences
# 
import sys, traceback
import time
from datetime import datetime
import selenium
from selenium import webdriver
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.by import By
from almaviva.logging import Logging
# import os     # fassad, 2021-08-28: removido
# from selenium.webdriver.common.keys import Keys   # fassad, 2021-08-29: removido
# from _library.filetemp import TempFiles   # fassad, 2021-08-29: removido
# from _library.debug import Debug   # fassad, 2021-08-28: removido

#
# Manipulador webdriver selenium
# 
class WebDriverChrome():

    def __init__ (
        self
    ,   debug=None      # removido
    ,   profile_location=None
    ,   executable_path='chromedriver'
    ):
        
        self.log = Logging()
        self.set_debugging (
            activated=False
        )

        # self.debug = debug   # fassad, 2021-08-28: removido
        self.executable_path = executable_path      # Chromium Driver
        self.options = webdriver.ChromeOptions()    # Importação das configurações do navegador

        if profile_location:
            # self.debug.logSQL ('inicialização do webdriver usando perfil da pasta {' + str(profile_location) + '}', str(__name__))   # fassad, 2021-08-28: removido
            # print (
            #     '> [WebDriverChrome.__init__]: ' + \
            #     'inicialização do webdriver usando perfil da pasta {' + str(profile_location) + '}'
            # )     # removido
            self.log.info (
                description='Executando navegador Google Chrome usando perfil localizado na pasta ' + str (
                    profile_location
                )
            ,   step='WebDriverChrome.__init__'
            )
            self.set_options('user-data-dir=' + str(profile_location))
        else:
            # self.debug.logSQL ('inicialização do webdriver usando perfil temporário padrão', str(__name__))   # fassad, 2021-08-28: removido
            # print (
            #     '> [WebDriverChrome.__init__]: ' + \
            #     'inicialização do webdriver usando perfil temporário padrão'
            # )     # removido
            self.log.warn (
                description='Executando navegador Google Chrome usando perfil temporário'
            ,   step='WebDriverChrome.__init__'
            )

    # 
    # Set debug-in-code
    # 
    def set_debugging (
        self
    ,   activated=False
    ):
        if (activated==True):
            self.debug = True
        else:
            self.debug = False

        self.log.set_options (
            debug_in_terminal=self.debug
        )

        if (self.debug==True):
            self.log.info (
                description='Modo de depuração habilitado'
            ,   step='WebDriverChrome.set_debugging'
            )
        else:
            self.log.warn (
                description='Modo de depuração desabilitado'
            ,   step='WebDriverChrome.set_debugging'
            )
            
    def set_options (
        self
    ,   new_argument
    ):
        self.options.add_argument (
            str (
                new_argument
            )
        )
        self.log.info (
            description='Opção \'' + str (
                new_argument
            ) + '\' foi adicionada nas configuraões do navegador'
        ,   step='WebDriverChrome.set_options'
        )

    def add_experimental_option (
        self
    ,   prefs=None
    ,   excludeSwitches=None
    ):
        if prefs:
            self.options.add_experimental_option (
                "prefs"
            ,   prefs
            )
            self.log.info (
                description='Opçõ(es) ' + str (
                    prefs
                ) + ' adicionad(as) nas configuraões do navegador'
            ,   step='WebDriverChrome.add_experimental_option'
            )
        if excludeSwitches:
            self.options.add_experimental_option (
                'excludeSwitches'
            ,   excludeSwitches
            )   # fassad, 2021-08-28: remove hardware error detection
            self.log.info (
                description='Opção [' + str (
                    excludeSwitches
                ) + '] adicionada nas configuraões do navegador'
            ,   step='WebDriverChrome.add_experimental_option'
            )
    
    # fassad, 2020-06-09: removido
    # def set_url (
    #     self
    # ,   url
    # ):
    #     self.url = str (
    #         url
    #     )

    def set_browser (
        self
    ,   url=None
    ):
        
        if 'browser' in dir (
            self
        ):
            if (url):
                # self.debug.logSQL ('abrindo página web {' + url + '}', str(__name__))   # fassad, 2021-08-28: removido
                # print (
                #     '> [WebDriverChrome.set_browser]: ' + \
                #     'processando: ' + url
                # )     # removido
                self.log.info (
                    description='Processando URL ' + str (
                        url
                    )
                ,   step='WebDriverChrome.set_browser'
                )
                self.browser.get (
                    url
                )
            else:
                # self.debug.logSQL ('abrindo nova página', str(__name__))   # fassad, 2021-08-28: removido
                # print (
                #     '> [WebDriverChrome.set_browser]: ' + \
                #     'processando'
                # )     # removido
                self.log.warn (
                    description='Navegador está em execução sem nenhuma URL definida'
                ,   step='WebDriverChrome.set_browser'
                )
                self.browser = webdriver.Chrome (
                    executable_path=self.executable_path
                ,   chrome_options=self.options
                )
                

        else:
            if (url):
                # self.debug.logSQL ('inicialização do navegador e processamento da página {' + url + '}', str(__name__))   # fassad, 2021-08-28: removido
                # print (
                #     '> [WebDriverChrome.set_browser]: ' + \
                #     'processando: ' + url
                # )     # removido
                self.log.info (
                    description='Processando URL ' + str (
                        url
                    )
                ,   step='WebDriverChrome.set_browser'
                )
                self.browser = webdriver.Chrome (
                    executable_path=self.executable_path
                ,   chrome_options=self.options
                )
                self.browser.get (
                    url
                )
            else:
                # self.debug.logSQL ('inicialização do navegador', str(__name__))   # fassad, 2021-08-28: removido
                # print (
                #     '> [WebDriverChrome.set_browser]: ' + \
                #     'processando'
                # )     # removido
                self.log.warn (
                    description='Navegador está em execução sem nenhuma URL definida'
                ,   step='WebDriverChrome.set_browser'
                )
                self.browser = webdriver.Chrome (
                    executable_path=self.executable_path
                ,   chrome_options=self.options
                )
        
        return self.browser

# 
# Manupulador para rescrita das funções find_element* selenium adicionando método em loop para otimizar run time
# 
class Browser (
    webdriver.Chrome
):

    def __init__ (
        self
    ,   debug=None  # # fassad, 2021-08-28: removido
    ,   browser=WebDriverChrome.set_browser
    ,   timeout=30
    ,   poll_frequency=1
    ):

        self.log = Logging()
        self.set_debugging (
            activated=False
        )

        # self.debug = debug   # fassad, 2021-08-28: removido
        self._browser = browser
        self._timeout = timeout
        self._poll = poll_frequency
    
    # 
    # Set debug-in-code
    # 
    def set_debugging (
        self
    ,   activated=False
    ):
        if (activated==True):
            self.debug = True
        else:
            self.debug = False

        self.log.set_options (
            debug_in_terminal=self.debug
        )

        if (self.debug==True):
            self.log.info (
                description='Modo de depuração habilitado'
            ,   step='Browser.set_debugging'
            )
        else:
            self.log.warn (
                description='Modo de depuração desabilitado'
            ,   step='Browser.set_debugging'
            )
    
    def quit (
        self
    ):
        self._browser.quit()
        # print (
        #     '> [Browser.quit]: ' + \
        #     'Ok'
        # )     # removido
        self.log.info (
            description='A aplicação web foi finalizada'
        ,   step='Browser.quit'
        )


    def refresh (
        self
    ):
        self._browser.refresh()
        # print (
        #     '> [Browser.refresh]: ' + \
        #     'Ok'
        # )     # removido
        self.log.info (
            description='A página web foi atualizada'
        ,   step='Browser.refresh'
        )

    def current_url (
        self
    ):
        self._browser.refresh()
        # print (
        #     '> [Browser.current_url]: ' + \
        #     self._browser.current_url
        # )     # removido
        self.log.info (
            description=str (
                self._browser.current_url
            )
        ,   step='Browser.current_url'
        )
        return self._browser.current_url

    def switchTo (
        self
    ,   tab=0
    ):
        # print (
        #     '> [Browser.switchTo]: ' + \
        #     self._browser.window_handles[tab]
        # )     # removido
        self.log.info (
            description='Alternando para guia nro. ' + str (
                tab
            ) + ' do navegador'
        ,   step='Browser.switchTo'
        )
        self._browser.switch_to.window (
            self._browser.window_handles [
                tab
            ]
        )

    def closeTab (
        self
    ,   tab=0
    ):
        self.switchTo (
            tab=tab
        )
        # print (
        #     '> [Browser.closeTab]: ' + \
        #     self._browser.window_handles [
        #         tab
        #     ]
        # )     # removido
        self.log.info (
            description='A guia nro. ' + str (
                tab
            ) + ' do navegador foi encerrado'
        ,   step='Browser.closeTab'
        )
        self._browser.close()

    # 
    # Esta função está desatualizada, em breve será removida. Utilize a função 'find_element()'
    # 
    def find_element_by_xpath (
        self
    ,   value
    ,   timeout=None
    ):

        
        try:
            
            # self.debug.logSQL ('localizar elemento da página usando { locate=xpath; value=' + str(value) + ' }', str(__name__))   # fassad, 2021-08-28: removido
            # print (
            #     '> [Browser.find_element_by_xpath]: ' + \
            #     'aguardando elemento web -> ' + str(value)
            # )     # remvido
            self.log.info (
                description='localizando XPATH > ' + str (
                    value
                )
            ,   step='Browser.find_element_by_xpath'
            )

            if (timeout):
                return WebDriverWait (
                    driver=value
                ,   timeout=timeout
                ,   poll_frequency=self._poll
                ).until (
                    method=self._browser.find_element_by_xpath
                )
            else:
                return WebDriverWait (
                    driver=value
                ,   timeout=self._timeout
                ,   poll_frequency=self._poll
                ).until (
                    method=self._browser.find_element_by_xpath
                )
            
        except Exception as ecp:
            # self.debug.logSQL ( \
            #     '! ' + str(sys.exc_info()[0]).replace('\'', '') + ' o processo para localizar o elemento da página usando { locate=xpath; value=' + str(value) + ' } falhou. o robo retornou ao processo anterior para evitar a interrupção do programa', \
            #     str(__name__) \
            # )   # fassad, 2021-08-28: removido
            # print (
            #     '> [Browser.find_element_by_xpath]: ' + \
            #     'Falhou -> ' + str(value)
            # )     # removido
            self.log.error (
                description='Falha ao localizar XPATH > ' + str (
                    value
                )
            ,   step='Browser.find_element_by_xpath'
            )
            print (
                sys.exc_info()
            )
            print (
                traceback.print_exc (
                    file=sys.stdout
                )
            )
            raise ecp

    def find_element (
        self
    ,   by=By
    ,   values=list
    ,   timeout=None
    ):

        # BY_CHOISES = {
        #     By.ID = "id"
        #     By.XPATH = "xpath"
        #     By.LINK_TEXT = "link text"
        #     By.PARTIAL_LINK_TEXT = "partial link text"
        #     By.NAME = "name"
        #     By.TAG_NAME = "tag name"
        #     By.CLASS_NAME = "class name"
        #     By.CSS_SELECTOR = "css selector"
        # }

        # self.debug.logSQL ('localizar elemento da página usando { locate=' + str(by) + '; values=' + str(values) + ' }', str(__name__))   # fassad, 2021-08-28: removido
        # print (
        #     '> [Browser.find_element]: ' + \
        #     'aguardando elemento web -> ' + str (
        #         values
        #     )
        # )     # removido
        self.log.info (
            description='Localizando ' + str (
                by
            ) + ' > ' + str (
                values
            )
        ,   step='Browser.find_element'
        )

        init_time = int (
            datetime.now().strftime (
                "%S"
            )
        )
        if timeout:
            end_time = init_time + timeout
        else:
            end_time = init_time + self._timeout
        poll_time = init_time
        
        while True:
            try:
                
                find = False
                for value in values:
                    try:
                        find = self._browser.find_element (
                            by=by
                        ,   value=value
                        ).is_displayed()
                    
                    except selenium.common.exceptions.NoSuchElementException:
                        continue
                    
                    except:
                        # print (
                        #     sys.exc_info()
                        # )
                        # print (
                        #     traceback.print_exc (
                        #         file=sys.stdout
                        #     )
                        # )     # removido
                        break
                    
                    else:
                        break
                
                if not find:
                    raise selenium.common.exceptions.NoSuchElementException
            
            except selenium.common.exceptions.NoSuchElementException:
                
                if poll_time < end_time:
                    poll_time = poll_time + self._poll
                    time.sleep (
                        self._poll
                    )
                    continue

                else:
                    # print (
                    #     '> [Browser.find_element]: ' + \
                    #     'Não encontrado -> ' + str (
                    #         values
                    #     )
                    # )     # removido
                    self.log.error (
                        description='Falhou ao localizar ' + str (
                            by
                        ) + ' > ' + str (
                            values
                        )
                    ,   step='Browser.find_element'
                    )
                    raise selenium.common.exceptions.TimeoutException
            
            except Exception as ecp:
                # self.debug.logSQL ( \
                #     '! ' + str(sys.exc_info()[0]).replace('\'', '') + ' o processo para localizar o elemento da página usando { locate=' + str(by) + '; value=' + str(value) + ' } falhou. o robo retornou ao processo anterior para evitar a interrupção do programa', \
                #     str(__name__) \
                # )   # fassad, 2021-08-28: removido
                # print (
                #     '> [Browser.find_element]: ' + \
                #     'Falhou -> ' + str (
                #         values
                #     )
                # )     # removido
                self.log.error (
                    description='Falhou ao localizar ' + str (
                        by
                    ) + ' > ' + str (
                        values
                    )
                ,   step='Browser.find_element'
                )
                print (
                    sys.exc_info()
                )
                print (
                    traceback.print_exc (
                        file=sys.stdout
                    )
                )
                raise ecp

            else:
                break
        
        self.log.info (
            description='Localizado ' + str (
                by
            ) + ' > ' + str (
                values
            )
        ,   step='Browser.find_element'
        )
        return self._browser.find_element (
            by=by
        ,   value=value
        )


# coding: utf-8

# 
# Dependences
# 
import time
from almaviva.ihm import HUNames    # fassad, 2021-09-11: adicionado na versão 1.2.0

# 
# Manipulador para registro do log
# 
class Logging():
    
    def __init__ (
        self
    ):
        
        self.set_options (
            # logging_in_db=None    # substituído na versão 1.2.0
            logging_in_file=True
        ,   debug_in_terminal=False
        )

        huname = HUNames()
        self.server = huname.get_hostname()
        self.user = huname.get_username()
        self.app = huname.get_app()

    #
    # Opções avançadas
    #
    def set_options (
        self
    # ,   logging_in_db=None    # substituído na versão 1.2.0
    ,   logging_in_file=True
    ,   debug_in_terminal=False
    ):
        
        # 
        # Regitra a atividade do servidor no banco de dados
        # 
        if (
            # logging_in_db   # informar a instância do objeto Database
            logging_in_file==True     # substituído na versão 1.2.0
        ):
            # self.logging_in_db = logging_in_db    # substituído na versão 1.2.0
            self.logging_in_file = True
        else:
            # self.logging_in_db = None     # substituído na versão 1.2.0
            self.logging_in_file = None
        
        #
        # Caso esteja habilitado, mostrará todos os registros no terminal a partir do nível de segurança 'INFO' até 'ERROR'
        # Case esteja desabilitado, exibirá apenas os registros a partir do nível 'WARNING' até 'ERROR', no terminal.
        # Esta opção não altera o modo de gravação dos registros no banco de dados
        #
        if (
            debug_in_terminal==True
        ):
            self.debug_in_terminal = True
        else:
            self.debug_in_terminal = None

    # 
    # Log do nível de segurança 'INFO' c/s registro em banco de dados
    # 
    def info (
        self
    ,   description
    ,   step=None
    ):
        now = time.time()
        mlsec = repr (
            now
        ).split (
            '.'
        )[1][:3]
        dateLocal = time.strftime (
            '%Y-%m-%d'
        ,   time.localtime (
                now
            )
        )
        dateUtc = time.strftime (
            '%Y-%m-%d'
        ,   time.gmtime()
        )
        timeLocal = time.strftime (
            '%H:%M:%S.{}'.format (
                mlsec
            )
        ,   time.localtime (
                now
            )
        )
        timeUtc = time.strftime (
            '%H:%M:%S.{}'.format (
                mlsec
            )
        ,   time.gmtime()
        )

        """
        contexto do log para exibição no terminal.
        contém os campos: dhLocal, level, step, description
        """
        log_in_terminal = \
            dateLocal \
        +   ' ' \
        +   timeLocal \
        +   ':' \
        +   'INFO' \
        +   ':' \
        +   str (
                step
            ).replace (
                ':'
            ,   '.'
            ) \
        +   ':' \
        +   str (
                description
            ).replace (
                ':'
            ,   '.'
            )

        """
        contexto do log para registro no arquivo.
        contém os campos: dhUtc, dhLocal, server, user, app, level, step, description
        """
        log_in_file = \
            dateUtc \
        +   ' ' \
        +   timeUtc \
        +   ':' \
        +   dateLocal \
        +   ' ' \
        +   timeLocal \
        +   ':' \
        +   str (
                self.server
            ).replace (
                ':'
            ,   '.'
            ) \
        +   ':' \
        +   str (
                self.user
            ).replace (
                ':'
            ,   '.'
            ) \
        +   ':' \
        +   str (
                self.app
            ).replace (
                ':'
            ,   '.'
            ) \
        +   ':' \
        +   'INFO' \
        +   ':' \
        +   str (
                step
            ).replace (
                ':'
            ,   '.'
            ) \
        +   ':' \
        +   str (
                description
            ).replace (
                ':'
            ,   '.'
            )

        """
        Verificar se a opção do registro do log em arquivo no sistema local foi habilitada
        """
        # if (self.logging_in_db):      # fassad, 2021-09-11: substituído na versão 1.2.0
        if (self.logging_in_file):
            
            """
            Mostrar registro de log resumido no terminal, caso a opção __debug_in_terminal__ esteja habilitada
            """
            if (self.debug_in_terminal==True):
                print ( \
                    '> '
                +   log_in_terminal \
                )

            _try=1
            _try_counts=10
            while True:

                try:

                    """
                    Registrar o log em arquivo no sistema local
                    """
                    with open (
                        file= \
                            dateLocal \
                        +   '.log'
                    ,   mode='a'
                    ,   encoding='utf8'
                    ) as fh:
                        readme = fh.write (
                            log_in_file + '\n'
                        )
                        fh.close()
                    
                    """
                    Tentar enviar o registro do log para o servidor de LOGS
                    """
                    # params = str(step), 'INFO', str(description)
                    # procedure = '[NocCentral].[dbo].[sp_ServersLogging]'
                    # param_tuple = (params)
                    # self.logging_in_db.execute_sp (
                    #     stored_procedure=procedure
                    # ,   param=param_tuple
                    # )     # fassad, 2021-09-11: substituído na versão 1.2.0
                    with open (
                        file='\\\\portalnoc\\logfiles$\\buffer.log'
                    ,   mode='a'
                    ,   encoding='utf8'
                    ) as fh:
                        readme = fh.write (
                            log_in_file + '\n'
                        )
                        fh.close()

                except FileNotFoundError as ecp:
                    # print (
                    #     sys.exc_info()
                    # )
                    # print (
                    #     traceback.print_exc (
                    #         file=sys.stdout
                    #     )
                    # )
                    break
                
                except PermissionError as ecp:
                    _try+=1
                    if (
                        _try > _try_counts
                    ):
                        raise ecp
                    
                    time.sleep (5)
                    continue

                else:
                    break

        # Desconsiderar o registro do log em arquivo no sistema local
        else:
            
            """
            Mostrar registro de log resumido no terminal, caso a opção __debug_in_terminal__ esteja habilitada
            """
            if (self.debug_in_terminal==True):
                print ( \
                    '> '
                +   log_in_terminal \
                )

            _try=1
            _try_counts=10
            while True:

                try:

                    """
                    Tentar enviar o registro do log para o servidor de LOGS
                    """
                    # params = str(step), 'INFO', str(description)
                    # procedure = '[NocCentral].[dbo].[sp_ServersLogging]'
                    # param_tuple = (params)
                    # self.logging_in_db.execute_sp (
                    #     stored_procedure=procedure
                    # ,   param=param_tuple
                    # )     # fassad, 2021-09-11: substituído na versão 1.2.0
                    with open (
                        file='\\\\portalnoc\\logfiles$\\buffer.log'
                    ,   mode='a'
                    ,   encoding='utf8'
                    ) as fh:
                        readme = fh.write (
                            log_in_file + '\n'
                        )
                        fh.close()

                except FileNotFoundError as ecp:
                    # print (
                    #     sys.exc_info()
                    # )
                    # print (
                    #     traceback.print_exc (
                    #         file=sys.stdout
                    #     )
                    # )
                    break
                
                except PermissionError as ecp:
                    _try+=1
                    if (
                        _try > _try_counts
                    ):
                        raise ecp
                    
                    time.sleep (5)
                    continue

                else:
                    break

    # 
    # Log do nível de segurança 'WARNING' c/s registro em banco de dados
    # 
    def warn (
        self
    ,   description
    ,   step=None
    ):
        now = time.time()
        mlsec = repr (
            now
        ).split (
            '.'
        )[1][:3]
        dateLocal = time.strftime (
            '%Y-%m-%d'
        ,   time.localtime (
                now
            )
        )
        dateUtc = time.strftime (
            '%Y-%m-%d'
        ,   time.gmtime()
        )
        timeLocal = time.strftime (
            '%H:%M:%S.{}'.format (
                mlsec
            )
        ,   time.localtime (
                now
            )
        )
        timeUtc = time.strftime (
            '%H:%M:%S.{}'.format (
                mlsec
            )
        ,   time.gmtime()
        )

        """
        contexto do log para exibição no terminal.
        contém os campos: dhLocal, level, step, description
        """
        log_in_terminal = \
            dateLocal \
        +   ' ' \
        +   timeLocal \
        +   ':' \
        +   'WARN' \
        +   ':' \
        +   str (
                step
            ).replace (
                ':'
            ,   '.'
            ) \
        +   ':' \
        +   str (
                description
            ).replace (
                ':'
            ,   '.'
            )

        """
        contexto do log para registro no arquivo.
        contém os campos: dhUtc, dhLocal, server, user, app, level, step, description
        """
        log_in_file = \
            dateUtc \
        +   ' ' \
        +   timeUtc \
        +   ':' \
        +   dateLocal \
        +   ' ' \
        +   timeLocal \
        +   ':' \
        +   str (
                self.server
            ).replace (
                ':'
            ,   '.'
            ) \
        +   ':' \
        +   str (
                self.user
            ).replace (
                ':'
            ,   '.'
            ) \
        +   ':' \
        +   str (
                self.app
            ).replace (
                ':'
            ,   '.'
            ) \
        +   ':' \
        +   'WARN' \
        +   ':' \
        +   str (
                step
            ).replace (
                ':'
            ,   '.'
            ) \
        +   ':' \
        +   str (
                description
            ).replace (
                ':'
            ,   '.'
            )

        """
        Verificar se a opção do registro do log em arquivo no sistema local foi habilitada
        """
        # if (self.logging_in_db):      # fassad, 2021-09-11: substituído na versão 1.2.0
        if (self.logging_in_file):
            
            """
            Mostrar registro de log resumido no terminal, caso a opção __debug_in_terminal__ esteja habilitada
            """
            if (self.debug_in_terminal==True):
                print ( \
                    '> '
                +   log_in_terminal \
                )

            _try=1
            _try_counts=10
            while True:

                try:

                    """
                    Registrar o log em arquivo no sistema local
                    """
                    with open (
                        file= \
                            dateLocal \
                        +   '.log'
                    ,   mode='a'
                    ,   encoding='utf8'
                    ) as fh:
                        readme = fh.write (
                            log_in_file + '\n'
                        )
                        fh.close()
                    
                    """
                    Tentar enviar o registro do log para o servidor de LOGS
                    """
                    # params = str(step), 'WARN', str(description)
                    # procedure = '[NocCentral].[dbo].[sp_ServersLogging]'
                    # param_tuple = (params)
                    # self.logging_in_db.execute_sp (
                    #     stored_procedure=procedure
                    # ,   param=param_tuple
                    # )     # fassad, 2021-09-11: substituído na versão 1.2.0
                    with open (
                        file='\\\\portalnoc\\logfiles$\\buffer.log'
                    ,   mode='a'
                    ,   encoding='utf8'
                    ) as fh:
                        readme = fh.write (
                            log_in_file + '\n'
                        )
                        fh.close()

                except FileNotFoundError as ecp:
                    # print (
                    #     sys.exc_info()
                    # )
                    # print (
                    #     traceback.print_exc (
                    #         file=sys.stdout
                    #     )
                    # )
                    break
                
                except PermissionError as ecp:
                    _try+=1
                    if (
                        _try > _try_counts
                    ):
                        raise ecp
                    
                    time.sleep (5)
                    continue

                else:
                    break

        # Desconsiderar o registro do log em arquivo no sistema local
        else:
            
            """
            Mostrar registro de log resumido no terminal, caso a opção __debug_in_terminal__ esteja habilitada
            """
            if (self.debug_in_terminal==True):
                print ( \
                    '> '
                +   log_in_terminal \
                )

            _try=1
            _try_counts=10
            while True:

                try:

                    """
                    Tentar enviar o registro do log para o servidor de LOGS
                    """
                    # params = str(step), 'WARN', str(description)
                    # procedure = '[NocCentral].[dbo].[sp_ServersLogging]'
                    # param_tuple = (params)
                    # self.logging_in_db.execute_sp (
                    #     stored_procedure=procedure
                    # ,   param=param_tuple
                    # )     # fassad, 2021-09-11: substituído na versão 1.2.0
                    with open (
                        file='\\\\portalnoc\\logfiles$\\buffer.log'
                    ,   mode='a'
                    ,   encoding='utf8'
                    ) as fh:
                        readme = fh.write (
                            log_in_file + '\n'
                        )
                        fh.close()

                except FileNotFoundError as ecp:
                    # print (
                    #     sys.exc_info()
                    # )
                    # print (
                    #     traceback.print_exc (
                    #         file=sys.stdout
                    #     )
                    # )
                    break
                
                except PermissionError as ecp:
                    _try+=1
                    if (
                        _try > _try_counts
                    ):
                        raise ecp
                    
                    time.sleep (5)
                    continue

                else:
                    break
    

    # 
    # Log do nível de segurança 'ERROR' c/s registro em banco de dados
    # 
    def error (
        self
    ,   description
    ,   step=None
    ):
        now = time.time()
        mlsec = repr (
            now
        ).split (
            '.'
        )[1][:3]
        dateLocal = time.strftime (
            '%Y-%m-%d'
        ,   time.localtime (
                now
            )
        )
        dateUtc = time.strftime (
            '%Y-%m-%d'
        ,   time.gmtime()
        )
        timeLocal = time.strftime (
            '%H:%M:%S.{}'.format (
                mlsec
            )
        ,   time.localtime (
                now
            )
        )
        timeUtc = time.strftime (
            '%H:%M:%S.{}'.format (
                mlsec
            )
        ,   time.gmtime()
        )

        """
        contexto do log para exibição no terminal.
        contém os campos: dhLocal, level, step, description
        """
        log_in_terminal = \
            dateLocal \
        +   ' ' \
        +   timeLocal \
        +   ':' \
        +   'ERROR' \
        +   ':' \
        +   str (
                step
            ).replace (
                ':'
            ,   '.'
            ) \
        +   ':' \
        +   str (
                description
            ).replace (
                ':'
            ,   '.'
            )

        """
        contexto do log para registro no arquivo.
        contém os campos: dhUtc, dhLocal, server, user, app, level, step, description
        """
        log_in_file = \
            dateUtc \
        +   ' ' \
        +   timeUtc \
        +   ':' \
        +   dateLocal \
        +   ' ' \
        +   timeLocal \
        +   ':' \
        +   str (
                self.server
            ).replace (
                ':'
            ,   '.'
            ) \
        +   ':' \
        +   str (
                self.user
            ).replace (
                ':'
            ,   '.'
            ) \
        +   ':' \
        +   str (
                self.app
            ).replace (
                ':'
            ,   '.'
            ) \
        +   ':' \
        +   'ERROR' \
        +   ':' \
        +   str (
                step
            ).replace (
                ':'
            ,   '.'
            ) \
        +   ':' \
        +   str (
                description
            ).replace (
                ':'
            ,   '.'
            )

        """
        Verificar se a opção do registro do log em arquivo no sistema local foi habilitada
        """
        # if (self.logging_in_db):      # fassad, 2021-09-11: substituído na versão 1.2.0
        if (self.logging_in_file):
            
            """
            Mostrar registro de log resumido no terminal, caso a opção __debug_in_terminal__ esteja habilitada
            """
            if (self.debug_in_terminal==True):
                print ( \
                    '> '
                +   log_in_terminal \
                )

            _try=1
            _try_counts=10
            while True:

                try:

                    """
                    Registrar o log em arquivo no sistema local
                    """
                    with open (
                        file= \
                            dateLocal \
                        +   '.log'
                    ,   mode='a'
                    ,   encoding='utf8'
                    ) as fh:
                        readme = fh.write (
                            log_in_file + '\n'
                        )
                        fh.close()
                    
                    """
                    Tentar enviar o registro do log para o servidor de LOGS
                    """
                    # params = str(step), 'ERROR', str(description)
                    # procedure = '[NocCentral].[dbo].[sp_ServersLogging]'
                    # param_tuple = (params)
                    # self.logging_in_db.execute_sp (
                    #     stored_procedure=procedure
                    # ,   param=param_tuple
                    # )     # fassad, 2021-09-11: substituído na versão 1.2.0
                    with open (
                        file='\\\\portalnoc\\logfiles$\\buffer.log'
                    ,   mode='a'
                    ,   encoding='utf8'
                    ) as fh:
                        readme = fh.write (
                            log_in_file + '\n'
                        )
                        fh.close()

                except FileNotFoundError as ecp:
                    # print (
                    #     sys.exc_info()
                    # )
                    # print (
                    #     traceback.print_exc (
                    #         file=sys.stdout
                    #     )
                    # )
                    break
                
                except PermissionError as ecp:
                    _try+=1
                    if (
                        _try > _try_counts
                    ):
                        raise ecp
                    
                    time.sleep (5)
                    continue

                else:
                    break

        # Desconsiderar o registro do log em arquivo no sistema local
        else:
            
            """
            Mostrar registro de log resumido no terminal, caso a opção __debug_in_terminal__ esteja habilitada
            """
            if (self.debug_in_terminal==True):
                print ( \
                    '> '
                +   log_in_terminal \
                )

            _try=1
            _try_counts=10
            while True:

                try:

                    """
                    Tentar enviar o registro do log para o servidor de LOGS
                    """
                    # params = str(step), 'ERROR', str(description)
                    # procedure = '[NocCentral].[dbo].[sp_ServersLogging]'
                    # param_tuple = (params)
                    # self.logging_in_db.execute_sp (
                    #     stored_procedure=procedure
                    # ,   param=param_tuple
                    # )     # fassad, 2021-09-11: substituído na versão 1.2.0
                    with open (
                        file='\\\\portalnoc\\logfiles$\\buffer.log'
                    ,   mode='a'
                    ,   encoding='utf8'
                    ) as fh:
                        readme = fh.write (
                            log_in_file + '\n'
                        )
                        fh.close()

                except FileNotFoundError as ecp:
                    # print (
                    #     sys.exc_info()
                    # )
                    # print (
                    #     traceback.print_exc (
                    #         file=sys.stdout
                    #     )
                    # )
                    break
                
                except PermissionError as ecp:
                    _try+=1
                    if (
                        _try > _try_counts
                    ):
                        raise ecp
                    
                    time.sleep (5)
                    continue

                else:
                    break


# coding: utf-8

# 
# Dependences
# 
import os, sys, traceback
import pyodbc
from almaviva.logging import Logging

# 
# Manipulador para conexão ao banco de dados
# 
class Database():
    
    def __init__ (
        self
    ,   server
    ,   database
    ,   username=None
    ,   password=None
    ,   driver=None
    ):

        self.log = Logging()
        self.set_debugging (
            activated=False
        )

        # set default DRIVER={ODBC Driver 13 for SQL Server} or private DRIVER={self.driver}
        if driver:
            self.set_driver (driver=driver)
        else:
            self.set_driver (driver='ODBC Driver 13 for SQL Server')

        self.set_server (server=server)
        self.set_database (database=database)
        self.set_username (username=username)
        self.set_password (password=password)

        # cursor init set
        self.cursor = list()

    # 
    # Set debug-in-code
    # 
    def set_debugging (
        self
    ,   activated=False
    ):
        if (activated==True):
            self.debug = True
        else:
            self.debug = False

        self.log.set_options (
            debug_in_terminal=self.debug
        )

        if (self.debug==True):
            self.log.info (
                description='Modo de depuração habilitado'
            ,   step='Database.set_debugging'
            )
        else:
            self.log.warn (
                description='Modo de depuração desabilitado'
            ,   step='Database.set_debugging'
            )

    # 
    # Set other driver
    # 
    def set_driver (
        self
    ,   driver
    ):
        self.driver = driver
        self.log.info (
            description='Driver configurado: ' + str (
                driver
            )
        ,   step='Database.set_driver'
        )

    # 
    # Set other server name
    # 
    def set_server (
        self
    ,   server
    ):
        self.server = server
        self.log.info (
            description='Servidor configurado: ' + str (
                server
            )
        ,   step='Database.set_server'
        )

    # 
    # Set other database name
    # 
    def set_database (
        self
    ,   database
    ):
        self.database = database
        self.log.info (
            description='Banco de dados configurado: ' + str (
                database
            )
        ,   step='Database.set_database'
        )

    # 
    # Set username name
    # 
    def set_username (
        self
    ,   username
    ):
        self.username = username
        self.log.info (
            description='Autenticação via username configurado'
        ,   step='Database.set_username'
        )

    # 
    # Set password name
    # 
    def set_password (
        self
    ,   password
    ):
        self.password = password
    
    #
    # Função para iniciar a conexão com banco de dados usando as informações da biblioteca 
    # 
    def connect (
        self
    ):
        
        try:
            if (self.username):
                self.log.info (
                    description='Iniciando conexão usando DRIVER=' + str(self.driver) + ';SERVER=' + str(self.server) + ';DATABASE=' + str(self.database) + ';Trusted_Connection=no'
                ,   step='Database.connect'
                )
                cnxn = pyodbc.connect ( \
                    'DRIVER={' + str(self.driver) + '};' \
                +   'SERVER=' + str(self.server) + ';' \
                +   'DATABASE=' + str(self.database) + ';' \
                +   'UID=' + str(self.username) + ';' \
                +   'PWD=' + str(self.password)
                )
            else:
                self.log.info (
                    description='Iniciando conexão usando DRIVER=' + str(self.driver) + ';SERVER=' + str(self.server) + ';DATABASE=' + str(self.database) + ';Trusted_Connection=yes'
                ,   step='Database.connect'
                )
                cnxn = pyodbc.connect ( \
                    'DRIVER={' + str(self.driver) + '};' \
                +   'SERVER=' + str(self.server) + ';' \
                +   'DATABASE=' + str(self.database) + ';' \
                +   'Trusted_Connection=yes'
                )   # add Trusted Connection

            self.cursor = cnxn.cursor()     # set cursor
            cnxn.autocommit = True
            self.log.info (
                description='Conectado'
            ,   step='Database.connect'
            )

            return True
        
        except Exception as ecp:
            self.log.error (
                description='Falhou'
            ,   step='Database.connect'
            )
            # return sys.exc_info()[0]      # alterado para permitir tratamento de falhas no código pai
            print (sys.exc_info())
            print (traceback.print_exc(file=sys.stdout))
            raise ecp
    
    #
    # SELECT
    # 
    def select (
        self
    ,   query
    ):
        
        try:
            self.cursor.execute(query)
            self.log.info (
                description='Ok > ' + str (
                    query
                )
            ,   step='Database.select'
            )
            return self.cursor.fetchall()

        except Exception as ecp:
            self.log.error (
                description='Falhou > ' + str (
                    query
                )
            ,   step='Database.select'
            )
            # return sys.exc_info()[0]      # alterado para permitir tratamento de falhas no código pai
            print (sys.exc_info())
            print (traceback.print_exc(file=sys.stdout))
            raise ecp

    #
    # INSERT
    # 
    def insert (
        self
    ,   query
    ):
        
        try:
            self.cursor.execute(query)
            self.log.info (
                description='Ok > ' + str (
                    query
                )
            ,   step='Database.insert'
            )
            return True

        except Exception as ecp:
            self.log.error (
                description='Falhou > ' + str (
                    query
                )
            ,   step='Database.insert'
            )
            # return sys.exc_info()[0]      # alterado para permitir tratamento de falhas no código pai
            print (sys.exc_info())
            print (traceback.print_exc(file=sys.stdout))
            raise ecp

    #
    # DELETE
    # 
    def delete (
        self
    ,   query
    ):
        
        try:
            self.cursor.execute(query)
            self.log.info (
                description='Ok > ' + str (
                    query
                )
            ,   step='Database.delete'
            )
            return True

        except Exception as ecp:
            self.log.error (
                description='Falhou > ' + str (
                    query
                )
            ,   step='Database.delete'
            )
            # return sys.exc_info()[0]      # alterado para permitir tratamento de falhas no código pai
            print (sys.exc_info())
            print (traceback.print_exc(file=sys.stdout))
            raise ecp

    #
    # UPDATE
    # 
    def update (
        self
    ,   query
    ):
        
        try:
            self.cursor.execute(query)
            self.log.info (
                description='Ok > ' + str (
                    query
                )
            ,   step='Database.update'
            )
            return True

        except Exception as ecp:
            self.log.error (
                description='Falhou > ' + str (
                    query
                )
            ,   step='Database.update'
            )
            # return sys.exc_info()[0]      # alterado para permitir tratamento de falhas no código pai
            print (sys.exc_info())
            print (traceback.print_exc(file=sys.stdout))
            raise ecp

    # 
    # STORED PROCEDURE
    # 
    def execute_sp (
        self
    ,   stored_procedure
    ,   param=None
    ):
        
        try:
            
            if (param):
                
                if not (type(param) == tuple):
                    
                    if (type(param) == str):
                        _param = '(\'' + str(param) + '\')'
                    
                    else:
                        _param = '(' + str(param) + ')'

                else:
                    _param = param      # fix issue
            
            else:

                # if (type(param) == int or type(param) == float):
                #     _param = '(' + str(param) + ')'
                # else:
                #     if (type(param) == str):
                #         _param = '(\'\')'
                #     else:
                #         _param = '()'
                _param = ''    # fix issue
            
            _query = '{call ' + str(stored_procedure) + str(_param) + '}'
            self.cursor.execute(_query)
            self.log.info (
                description='Ok > ' + str (
                    _query
                )
            ,   step='Database.execute_sp'
            )
        
        except Exception as ecp:
            self.log.error (
                description='Falhou > ' + str (
                    _query
                )
            ,   step='Database.execute_sp'
            )
            print (sys.exc_info())
            print (traceback.print_exc(file=sys.stdout))
            raise ecp


# coding: utf-8
'''
Informações sobre o pacote
'''

#
# Dependencies
#
from setuptools import setup

#
# Link to README.md file
#
with open (
    file="README.md"
,   mode="r"
,   encoding="utf8"
) as fh:
    readme = fh.read()

#
# Setup
#
setup (
    name='almaviva'
,   version='1.2.0'
,   url='https://github.com/Almaviva-do-Brasil/almaviva'
,   packages=[
        'almaviva/database'
    ,   'almaviva/file'
    ,   'almaviva/ihm'
    ,   'almaviva/logging'
    ,   'almaviva/webdriver'
    ]
,   install_requires=[
        'pyodbc'
    ,   'wmi'
    ,   'selenium'
    ]
,   author='Assad, Felipe'
,   author_email='fassad@almavivadobrasil.com.br'
,   description=u'Biblioteca com funções personalizadas desenvolvidos para Almaviva Do Brasil'
,   license='Almaviva do Brasil Licence'
,   keywords='almavivadobrasil, database, logging, file, ihm, webdriver'
,   long_description=readme
,   long_description_content_type="text/markdown"
)